package org.johnchamberlain.chess;

import javax.microedition.lcdui.*;

public class DisplayView_Board extends Canvas implements CommandListener {

	public static final int BOARD_Empty        = 0x1F;
	public static final int SQUARE_White       = 0x00;
	public static final int SQUARE_Black       = 0x10;
	public static final int PIECE_ColorMask    = 0x8;
	public static final int PIECE_ColorWhite   = 0x0;
	public static final int PIECE_ColorBlack   = 0x8;
	public static final int PIECE_TypeMask     = 0x7;
	public static final int PIECE_None         = 0x0;
	public static final int PIECE_Pawn         = 0x1;
	public static final int PIECE_Knight       = 0x2;
	public static final int PIECE_Bishop       = 0x3;
	public static final int PIECE_Rook         = 0x4;
	public static final int PIECE_Queen        = 0x5;
	public static final int PIECE_King         = 0x6;
	public static final int PIECE_PawnWhite    = 0x1;
	public static final int PIECE_KnightWhite  = 0x2;
	public static final int PIECE_BishopWhite  = 0x3;
	public static final int PIECE_RookWhite    = 0x4;
	public static final int PIECE_QueenWhite   = 0x5;
	public static final int PIECE_KingWhite    = 0x6;
	public static final int PIECE_PawnBlack    = 0x9;
	public static final int PIECE_KnightBlack  = 0xA;
	public static final int PIECE_BishopBlack  = 0xB;
	public static final int PIECE_RookBlack    = 0xC;
	public static final int PIECE_QueenBlack   = 0xD;
	public static final int PIECE_KingBlack    = 0xE;
	public static final int PIECE_Max          = 0xE;
	public static final int PIECE_Promotion_Queen     = 0x0;
	public static final int PIECE_Promotion_Rook      = 0x1;
	public static final int PIECE_Promotion_Bishop    = 0x2;
	public static final int PIECE_Promotion_Knight    = 0x3;

	private static final int ANCHOR_TopLeft = Graphics.TOP | Graphics.LEFT;

    public DisplayView_Board() {}

	org.johnchamberlain.chess.DisplayModel_Board theBoardModel;

	private static final int SQUARE_FileMask  = 0x0F;
	private static final int SQUARE_RankShift = 4;
    private static final int PIXEL_SIZE_BoardSquare = 20;
    private static final int PIXEL_SIZE_BoardBorder = 0;

    static final int ANNOTATION_Select = 0x00000001;

    private static StringBuffer sbDisplayError = new StringBuffer(256);

    private Image imgDisplayBuffer = null;         //used to buffer the screen
    private Graphics gDisplayBuffer;

    //private static final int LOCUS_Board   = 1;
    //private static final int LOCUS_Moves   = 2;
    //private static final int LOCUS_Caption = 3;

    //private final Point[] apointScreenLocus = new Point[3 +1];

    private Image[] imgPiece       = new Image[0x1F +1];
    private int[] pixelTop         = new int[0x88 +1];
    private int[] pixelLeft        = new int[0x88 +1];
    private int[] aiSquareColor    = new int[0x88 +1]; // 0 - White, 1 - Black
    private int[] aiSquareMappings = new int[0x88 +1];

    boolean zInitialize(DisplayModel_Board theBoardModel, StringBuffer sbError){

        try{

			if(theBoardModel == null){
				sbError.append("board model was null");
				return false;
			} else {
				this.theBoardModel = theBoardModel;
			}

            if(!zLoadPieceImages(sbError)){
				sbError.insert(0, "unable to load images: ");
				return false;
			}

            vLoadPixelPositions();
            vLoadSquareColor();

            return true;
        }
        catch(Exception ex){
            sbError.insert(0, "Unexpected error: " + ex);
            return false;
        }
    }

    private void vLoadSquareColor(){
        int xRank, xFile;
        int xColor=SQUARE_Black;
        for(xRank=1; xRank<=8; xRank++){
            for(xFile=1; xFile<=8; xFile++){
                int xSquare = (xRank << SQUARE_RankShift) + xFile;
                aiSquareColor[xSquare]=xColor;
                if(xColor==SQUARE_Black) xColor=SQUARE_White; else xColor=SQUARE_Black;
            }
            if(xColor==SQUARE_Black) xColor=SQUARE_White; else xColor=SQUARE_Black;
        }
    }

    private boolean zLoadPieceImages(StringBuffer sbError){
		try {
			String sPath;
			sPath = "/images/";
			imgPiece[BOARD_Empty] = Image.createImage(sPath + "diagram_empty.gif");   //extra piece
			sPath += "piece_";  // "/images/piece_"
			imgPiece[PIECE_None] = Image.createImage(sPath + "squarew_empty.gif");  // not used
			imgPiece[PIECE_PawnWhite] = Image.createImage(sPath + "pawn_wow.gif");
			imgPiece[PIECE_KnightWhite] = Image.createImage(sPath + "knight_wow.gif");
			imgPiece[PIECE_BishopWhite] = Image.createImage(sPath + "bishop_wow.gif");
			imgPiece[PIECE_RookWhite] = Image.createImage(sPath + "rook_wow.gif");
			imgPiece[PIECE_QueenWhite] = Image.createImage(sPath + "queen_wow.gif");
			imgPiece[PIECE_KingWhite] = Image.createImage(sPath + "king_wow.gif");
			imgPiece[PIECE_PawnBlack] = Image.createImage(sPath + "pawn_bow.gif");
			imgPiece[PIECE_KnightBlack] = Image.createImage(sPath + "knight_bow.gif");
			imgPiece[PIECE_BishopBlack] = Image.createImage(sPath + "bishop_bow.gif");
			imgPiece[PIECE_RookBlack] = Image.createImage(sPath + "rook_bow.gif");
			imgPiece[PIECE_QueenBlack] = Image.createImage(sPath + "queen_bow.gif");
			imgPiece[PIECE_KingBlack] = Image.createImage(sPath + "king_bow.gif");
			imgPiece[PIECE_None | SQUARE_Black] = Image.createImage(sPath + "squareb_empty.gif");
			imgPiece[PIECE_PawnWhite | SQUARE_Black] = Image.createImage(sPath + "pawn_wob.gif");
			imgPiece[PIECE_KnightWhite | SQUARE_Black] = Image.createImage(sPath + "knight_wob.gif");
			imgPiece[PIECE_BishopWhite | SQUARE_Black] = Image.createImage(sPath + "bishop_wob.gif");
			imgPiece[PIECE_RookWhite | SQUARE_Black] = Image.createImage(sPath + "rook_wob.gif");
			imgPiece[PIECE_QueenWhite | SQUARE_Black] = Image.createImage(sPath + "queen_wob.gif");
			imgPiece[PIECE_KingWhite | SQUARE_Black] = Image.createImage(sPath + "king_wob.gif");
			imgPiece[PIECE_PawnBlack | SQUARE_Black] = Image.createImage(sPath + "pawn_bob.gif");
			imgPiece[PIECE_KnightBlack | SQUARE_Black] = Image.createImage(sPath + "knight_bob.gif");
			imgPiece[PIECE_BishopBlack | SQUARE_Black] = Image.createImage(sPath + "bishop_bob.gif");
			imgPiece[PIECE_RookBlack | SQUARE_Black] = Image.createImage(sPath + "rook_bob.gif");
			imgPiece[PIECE_QueenBlack | SQUARE_Black] = Image.createImage(sPath + "queen_bob.gif");
			imgPiece[PIECE_KingBlack | SQUARE_Black] = Image.createImage(sPath + "king_bob.gif");
			return true;
		} catch(Exception ex) {
			Utility.errorUnexpected(ex, sbError);
			return false;
		}
    }

    private void vLoadPixelPositions(){
        for(int xRank=1; xRank<=8; xRank++){
            for(int xFile=1; xFile<=8; xFile++){
                int xSquare = (xRank << SQUARE_RankShift) + xFile;
	            pixelTop[xSquare] = (8 - xRank) * PIXEL_SIZE_BoardSquare + PIXEL_SIZE_BoardBorder;
                pixelLeft[xSquare] = (xFile - 1) * PIXEL_SIZE_BoardSquare + PIXEL_SIZE_BoardBorder;
			}
        }
    }

    protected void paint(javax.microedition.lcdui.Graphics g){

        if(imgDisplayBuffer==null){
            imgDisplayBuffer = Image.createImage(160, 160);
            if(imgDisplayBuffer == null){
				System.err.println("Could not create blank image 160x160 for buffer.");
				return;
			}
	        gDisplayBuffer = imgDisplayBuffer.getGraphics();
		}

		//draw board in buffer
		gDisplayBuffer.drawImage(imgPiece[BOARD_Empty], 0, 0, ANCHOR_TopLeft);   //draw the board
		vRepaintBoard(gDisplayBuffer);

	    //blit buffer to screen
    	g.drawImage(imgDisplayBuffer, 0, 0, ANCHOR_TopLeft);

    }

    private void vRepaintBoard(Graphics g){
        for(int xRank=1; xRank<=8; xRank++){
            for(int xFile=1; xFile<=8; xFile++){
                int xSquare = (xRank << SQUARE_RankShift) + xFile;
                vRepaintSquare(g, xSquare);
			}
        }
    }

	final static int[] theEmptyBoard = new int[0x88 +1];

    private void vRepaintSquare(Graphics g, int iSquare){
        int xImage;
		int[] theBoard;

		if(theBoardModel.thePosition==null){ // then the board is empty
			theBoard = theEmptyBoard;
			return;
		} else {
            theBoard = theBoardModel.thePosition.aiBoard;
		}

        // draw squares
        if(theBoard[iSquare] > 0){
            xImage = aiSquareColor[iSquare] | theBoard[iSquare];
            g.drawImage(imgPiece[xImage], pixelLeft[iSquare], pixelTop[iSquare], ANCHOR_TopLeft);}
        else{
            xImage = aiSquareColor[iSquare] + 13;}

    }


}

