package org.johnchamberlain.chess;

public class Position {

//Notes (see position engine for more information)
//Pieces (note that Piece Type is the white column only):
//       White   Black
//Pawn   1       7
//Knight 2       8
//Bishop 3       9
//Rook   4       10
//Queen  5       11
//King   6       12

	private static final int SQUARE_FileMask  = 0x0F;
	private static final int SQUARE_RankShift = 4;

    public Position() {}

    int[] aiBoard = new int[0x88];
    boolean zWhiteToMove = true;
    boolean zKingMovedW, zKingMovedB;       // significant for castling
    boolean zKRookMovedW, zKRookMovedB;     // significant for castling
    boolean zQRookMovedW, zQRookMovedB;     // significant for castling
    int mvLastFrom, mvLastTo;               // 0 for initial position (significant for en passant)
    int ctMovesSinceLastPawnMove;           // significant for 50-move rule - count is ply

	public static final int PIECE_None    = 0x00000000;
	public static final int PIECE_Unknown = 0x000000;
	public static final int PIECE_Pawn_NoShift    = 0x1000000; // white
	public static final int PIECE_Pawn    = 0x1; // white
	public static final int PIECE_Knight  = 0x2; // white
	public static final int PIECE_Bishop  = 0x3; // white
	public static final int PIECE_Rook    = 0x4; // white
	public static final int PIECE_Queen   = 0x5; // white
	public static final int PIECE_King    = 0x6; // white
	public static final int PIECE_BPawn   = 0x9;
	public static final int PIECE_BKnight = 0xA;
	public static final int PIECE_BBishop = 0xB;
	public static final int PIECE_BRook   = 0xC;
	public static final int PIECE_BQueen  = 0xD;
	public static final int PIECE_BKing   = 0xE;

	/** clones the position */
	Position rclone(){
		Position positionNew = new Position();
		positionNew.aiBoard = new int[137];
		System.arraycopy(this.aiBoard, 0, positionNew.aiBoard, 0, 137);
		positionNew.zWhiteToMove = this.zWhiteToMove;
		positionNew.zKingMovedW  = this.zKingMovedW;
		positionNew.zKingMovedB  = this.zKingMovedB;
		positionNew.zKRookMovedW = this.zKRookMovedW;
		positionNew.zKRookMovedB = this.zKRookMovedB;
		positionNew.zQRookMovedW = this.zQRookMovedW;
		positionNew.zQRookMovedB = this.zQRookMovedB;
		positionNew.mvLastFrom   = this.mvLastFrom;
		positionNew.mvLastTo     = this.mvLastTo;
		positionNew.ctMovesSinceLastPawnMove = this.ctMovesSinceLastPawnMove;
		return positionNew;
	}

	public String toString(){
		StringBuffer sb = new StringBuffer(64);
        for(int xRank=8; xRank>0; xRank--){
            for(int xFile=1; xFile<=8; xFile++){
                int xSquare = (xRank << SQUARE_RankShift) + xFile;
				switch(aiBoard[xSquare]){
					case PIECE_None    : sb.append("."); break;
					case PIECE_King    : sb.append("K"); break;
					case PIECE_Queen   : sb.append("Q"); break;
					case PIECE_Rook    : sb.append("R"); break;
					case PIECE_Bishop  : sb.append("B"); break;
					case PIECE_Knight  : sb.append("N"); break;
					case PIECE_Pawn    : sb.append("P"); break;
					case PIECE_BKing   : sb.append("k"); break;
					case PIECE_BQueen  : sb.append("q"); break;
					case PIECE_BRook   : sb.append("r"); break;
					case PIECE_BBishop : sb.append("b"); break;
					case PIECE_BKnight : sb.append("n"); break;
					case PIECE_BPawn   : sb.append("p"); break;
				}
			}
			sb.append("\n");
		}
		return sb.toString();
	}
}


