package org.johnchamberlain.j2meframework.client;

/**
 * Title:        Live Chess Client
 * Description:  Connects to a Live Chess server
 * Copyright:    Copyright (c) 2002
 * @author John Chamberlain
 * @version 1.0
 */

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import org.johnchamberlain.chess.*;
import org.johnchamberlain.j2meframework.DataPacket;
import java.io.*;

public class LiveChessClient extends MIDlet {

	private DisplayView_Board canvasBoard;
    private Display mDisplay;
	TextBox tbMain;
	private Command commandExit;

	public LiveChessClient() {
		mDisplay = Display.getDisplay(this);
	}

	public void startApp(){
		canvasBoard = new DisplayView_Board();
		DisplayModel_Board dmBoard = new DisplayModel_Board();
		StringBuffer sbError = new StringBuffer(250);
		if( canvasBoard.zInitialize( dmBoard, sbError ) ){
			mDisplay.setCurrent(canvasBoard);
		} else {
			System.err.println(sbError);
		}
	}

	public void pauseApp(){}

	public void destroyApp(boolean z){}

	public void commandAction(Command c, Displayable d){
		if(c == commandExit){
			this.destroyApp(false);
			this.notifyDestroyed();
		}
	}

	static boolean zValidatePacket( DataPacket dp ){
		return true;
	}

	static void vProcessPacket( DataPacket dp ){
	}

}

class ServerCommunicator {

	// override these methods to provide functionality
	boolean zPacketArrived( DataPacket dp ){ return false; }
	boolean zPacketSend( DataPacket dp ){ return false; }
}

class ServerCommunicator_Polling extends ServerCommunicator {
	private final static int POLLING_INTERVAL_MS = 10000;
	private final static String SERVER_URL = "http://example.com/data-servlet";
	void vStart(){
		PollingContainer poller = new PollingContainer(this);
	    java.util.Timer timerPoller = new java.util.Timer();
	    timerPoller.schedule(poller, 0, POLLING_INTERVAL_MS);
	}
	String getURL(){ return SERVER_URL; }
	boolean zPacketArrived(DataPacket dp){
		if( !LiveChessClient.zValidatePacket(dp) ) return false;
		LiveChessClient.vProcessPacket(dp);
	}
	boolean zPacketSend( DataPacket dp ){
		try {
			int iDataLength = dp.length();
			HttpConnection con = (HttpConnection)Connector.open(this.getURL());
			con.setRequestMethod(HttpConnection.POST);
			con.setRequestProperty("User-Agent", "Profile/MIDP-1.0 Configuration/CLDC-1.0" );
			con.setRequestProperty("Content-Language", "en-US" );
			con.setRequestProperty("Accept", "application/octet-stream" );
			con.setRequestProperty("Connection", "close" );
			con.setRequestProperty("Content-Length", Integer.toString( iDataLength ) );
			OutputStream os = con.openOutputStream();
			os.write( dp.getBytes() );
			os.close();
			return true;
		} catch( Exception ex ) {
			return false;
		}
	}
}

class PollingContainer extends java.util.TimerTask {
	ServerCommunicator_Polling mParent;
	public PollingContainer( ServerCommunicator_Polling parent ){
	    mParent = parent;
	}
	public final void run(){
	    String sURL = mParent.getURL();
		try {
			vPollServer( sURL );
		} catch(Exception ex) {}
	}
	public void vPollServer ( String sURL ) throws IOException {
		HttpConnection con = (HttpConnection)Connector.open(sURL);
		con.setRequestMethod(HttpConnection.GET);
		con.setRequestProperty("User-Agent", "Profile/MIDP-1.0 Configuration/CLDC-1.0" );
		con.setRequestProperty("Content-Language", "en-US" );
		con.setRequestProperty("Accept", "application/octet-stream" );
		con.setRequestProperty("Connection", "close" );
		byte[] abData;
		if( con.getResponseCode() == HttpConnection.HTTP_OK ){
			InputStream isData = con.openInputStream();
			int iDataLength = (int)con.getLength();
			if( iDataLength == -1 ){ // read one char at a time
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				int iData;
				while( (iData = isData.read()) != -1 )
					baos.write(iData);
				abData = baos.toByteArray();
				baos.close();
			} else { // length is known, read all at once
				abData = new byte[iDataLength];
				isData.read( abData );
			}
		}
		DataPacket dpArrived = new DataPacket_LiveChess(abData);
		mParent.zPacketArrived( dpArrived );
	}
}




